@@ -32,7 +32,7 @@
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.15'
+__version__ = '0.16'
 
 
 # These are reserved ipv6 addresses.
@@ -68,6 +68,9 @@ ACCESS_RIGHTS = ['rw', 'r']
 # it is important not to overwrite these tables when doing a partial replay
 TABLES_NOT_AUDIT_LOGGED = ['audit_log', 'locks']
 
+# This is a list of ronly ecord types that can be modified by user level.
+USER_LEVEL_RECORDS = ['a', 'aaaa', 'ptr', 'cname']
+
 # This is a list of record types that can be indexed by IP address.
 RECORD_TYPES_INDEXED_BY_IP = ['ptr', 'a', 'aaaa']
 
@@ -79,11 +82,12 @@ SERVER_CONFIG_FILE_LOCATION = '/etc/roster/roster_server.conf'
 # 'write' indicates whether the method requires write access.
 # 'access_level' is the minimum user access level required to use this method.
 SUPPORTED_METHODS = {
-    'noop':         {'check': False,
+    'ListRecords':  {'check': False,
                      'write': False,
-                     'access_level': ACCESS_LEVELS['noop']},
+                     'access_level': ACCESS_LEVELS['user']},
 
-    'ListRecords':  {'check': True,
+    'ListRecordsByCIDRBlock':
+                    {'check': False,
                      'write': False,
                      'access_level': ACCESS_LEVELS['user']},
 
@@ -116,6 +120,11 @@ SUPPORTED_METHODS = {
                      'write': False,
                      'access_level': ACCESS_LEVELS['domain_admin']},
 
+    'GetViewsByUser':
+                    {'check': False,
+                     'write': False,
+                     'access_level': ACCESS_LEVELS['user']},
+
     'MakeView':     {'check': False,
                      'write': True,
                      'access_level': ACCESS_LEVELS['dns_admin']},
@@ -213,6 +222,16 @@ SUPPORTED_METHODS = {
                      'write': True,
                      'access_level': ACCESS_LEVELS['dns_admin']},
 
+    'GetAssociatedCNAMEs':
+                    {'check': False,
+                     'write': False,
+                     'access_level': ACCESS_LEVELS['user']},
+
+    'RemoveCNamesByAssignmentHost':
+                    {'check': True,
+                     'write': True,
+                     'access_level': ACCESS_LEVELS['user']},
+
     'ListACLs':     {'check': False,
                      'write': False,
                      'access_level': ACCESS_LEVELS['domain_admin']},
@@ -290,6 +309,16 @@ SUPPORTED_METHODS = {
                      'write': True,
                      'access_level': ACCESS_LEVELS['dns_admin']},
 
+    'GetCIDRBlocksByView':
+                    {'check': False,
+                     'write': False,
+                     'access_level': ACCESS_LEVELS['user']},
+
+    'ProcessRecordsBatch':
+                    {'check': True,
+                     'write': True,
+                     'access_level': ACCESS_LEVELS['user']},
+
     'ListZones':    {'check': False,
                      'write': False,
                      'access_level': ACCESS_LEVELS['domain_admin']},
@@ -354,12 +383,12 @@ SUPPORTED_METHODS = {
                      'access_level': ACCESS_LEVELS['dns_admin']},
 
     'ListNamedConfGlobalOptions':
-                    {'check': True,
+                    {'check': False,
                      'write': False,
                      'access_level': ACCESS_LEVELS['dns_admin']},
 
     'MakeNamedConfGlobalOption':
-                    {'check': True,
+                    {'check': False,
                      'write': True,
                      'access_level': ACCESS_LEVELS['dns_admin']},
 
@@ -382,30 +411,31 @@ SUPPORTED_METHODS = {
                     {'check': False,
                      'write': True,
                      'access_level': ACCESS_LEVELS['dns_admin']},
+
     'ListAuditLog':
                     {'check': False,
                      'write': False,
                      'access_level': ACCESS_LEVELS['dns_admin']},
+
     'ListCredentials':
                     {'check': False,
                      'write': False,
                      'access_level': ACCESS_LEVELS['dns_admin']},
+
     'MakeInfiniteCredential':
                     {'check': False,
                      'write': True,
                      'access_level': ACCESS_LEVELS['dns_admin']},
+
     'RemoveCredential':
                     {'check': False,
                      'write': True,
                      'access_level': ACCESS_LEVELS['dns_admin']},
+
     'SetMaintenanceFlag':
                     {'check': False,
                      'write': True,
-                     'access_level': ACCESS_LEVELS['dns_admin']},
-    'CheckMaintenanceFlag':
-                    {'check': False,
-                     'write': False,
-                     'access_level': ACCESS_LEVELS['user']}}
+                     'access_level': ACCESS_LEVELS['dns_admin']}}
 
 
 
@@ -445,7 +475,7 @@ TABLES = {
 
     'records':           {'records_id': 'UnsignedInt',
                           'record_type': 'UnicodeString',
-                          'record_target': 'UnicodeString',
+                          'record_target': 'UnicodeStringNoSpaces',
                           'record_ttl': 'UnsignedInt',
                           'record_zone_name': 'UnicodeString',
                           'record_view_dependency': 'UnicodeString',
