@@ -32,18 +32,15 @@
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.15'
+__version__ = '0.16'
 
 
 import constants
-import core
 import errors
 import helpers_lib
-import user
 
 import datetime
-import math
-
+import dns.zone
 import IPy
 
 class CoreHelpers(object):
@@ -70,24 +67,28 @@ class CoreHelpers(object):
   def UnReverseIP(self, ip_address):
     return helpers_lib.UnReverseIP(ip_address)
 
-  def CIDRExpand(self, cidr_block):
-    return helpers_lib.CIDRExpand(cidr_block)
+  def CIDRExpand(self, cidr_block, begin=None, end=None):
+    return helpers_lib.CIDRExpand(cidr_block, begin, end)
 
   def ExpandIPV6(self, ip_address):
     return helpers_lib.ExpandIPV6(ip_address)
 
-  def GetViewsByUser(self, user):
-    """Lists view names available to given user
+  def GetViewsByUser(self, username):
+    """Lists view names available to given username
 
     Inputs:
-        user: string of user name
+        username: string of user name
 
     Outputs:
         list: list of view name strings
     """
+    function_name, current_args = helpers_lib.GetFunctionNameAndArgs()
+    self.user_instance.Authorize(function_name)
     views = set([])
+    success = False
+
     users_dict = self.db_instance.GetEmptyRowDict('users')
-    users_dict['user_name'] = user
+    users_dict['user_name'] = username
     user_group_assignments_dict = self.db_instance.GetEmptyRowDict(
         'user_group_assignments')
     groups_dict = self.db_instance.GetEmptyRowDict('groups')
@@ -109,21 +110,155 @@ class CoreHelpers(object):
     for view_dict in joined_list:
       views.add(view_dict['zone_view_assignments_view_dependency'].split(
           '_dep')[0])
+    success = True
+
     return views
 
-  def GetCIDRBlocksByView(self, view, user):
-    """Lists CIDR blocks available to given view
+  def _FixHostname(self, host_name, origin):
+    """Checks name and returns fqdn.
+
+    Inputs:
+      host_name: string of host name
+      origin: string of the zone origin
+
+    Outputs:
+      string of fully qualified domain name
+    """
+    if( host_name == u'@' ):
+      host_name = origin
+    elif( not host_name.endswith('.') ):
+      host_name = '%s.%s' % (host_name, origin)
+    return unicode(host_name)
+
+  def AddFormattedRecords(self, zone_name, zone_file_string,
+                          view=u'any', zone_view=None,
+                          use_soa=True):
+    """Adds records from a string of a partial zone file
+
+    Inputs:
+      zone_name: string of zone name
+      zone_file_string: string of zone name
+      view: string of view name
+      zone_view: view to put SOA record in
+      use_soa: use a zone file with an SOA record
+
+    Outputs:
+      int: Amount of records added to db.
+    """
+    if( use_soa ):
+      zone = dns.zone.from_text(str(zone_file_string))
+      origin = zone.origin
+    else:
+      origin = self.core_instance.ListZones(zone_name=zone_name)[
+          zone_name][view][u'zone_origin']
+      zone = dns.zone.from_text(
+          str(zone_file_string), check_origin=False, origin=origin)
+    make_record_args_list = []
+    for record_tuple in zone.items():
+      record_target = unicode(record_tuple[0])
+
+      for record_set in record_tuple[1].rdatasets:
+        ttl = record_set.ttl
+        for record_object in record_set.items:
+          if( record_object.rdtype == dns.rdatatype.PTR ):
+            record_type = u'ptr'
+            assignment_host = self._FixHostname(unicode(record_object), origin)
+            record_args_dict = {u'assignment_host': assignment_host}
+
+          elif( record_object.rdtype == dns.rdatatype.A ):
+            record_type = u'a'
+            record_args_dict = {u'assignment_ip': unicode(record_object)}
+
+          elif( record_object.rdtype == dns.rdatatype.AAAA ):
+            record_type = u'aaaa'
+            record_args_dict = {u'assignment_ip':
+                                    unicode(IPy.IP(str(
+                                        record_object)).strFullsize())}
+
+          elif( record_object.rdtype == dns.rdatatype.CNAME ):
+            record_type = u'cname'
+            assignment_host = self._FixHostname(unicode(record_object), origin)
+            record_args_dict = {u'assignment_host': assignment_host}
+
+          elif( record_object.rdtype == dns.rdatatype.HINFO ):
+            record_type = u'hinfo'
+            record_args_dict = {u'hardware': unicode(record_object.cpu),
+                                u'os': unicode(record_object.os)}
+
+          elif( record_object.rdtype == dns.rdatatype.TXT ):
+            record_type = u'txt'
+            record_args_dict = {u'quoted_text': unicode(record_object)}
+
+          elif( record_object.rdtype == dns.rdatatype.MX ):
+            record_type = u'mx'
+            mail_server = self._FixHostname(unicode(record_object.exchange), origin)
+            record_args_dict = {u'priority': record_object.preference,
+                                u'mail_server': mail_server}
+
+          elif( record_object.rdtype == dns.rdatatype.NS ):
+            record_type = u'ns'
+            name_server = self._FixHostname(unicode(record_object), origin)
+            record_args_dict = {u'name_server': name_server}
+
+          elif( record_object.rdtype == dns.rdatatype.SRV ):
+            record_type = u'srv'
+            assignment_host = self._FixHostname(unicode(record_object.target), origin)
+            record_args_dict = {u'priority': record_object.priority,
+                                u'weight': record_object.weight,
+                                u'port': record_object.port,
+                                u'assignment_host': assignment_host}
+
+          elif( record_object.rdtype == dns.rdatatype.SOA ):
+            record_type = u'soa'
+            name_server = self._FixHostname(unicode(record_object.mname), origin)
+            admin_email = self._FixHostname(unicode(record_object.rname), origin)
+            record_args_dict = {u'name_server': name_server,
+                                u'admin_email': admin_email,
+                                u'serial_number': record_object.serial,
+                                u'retry_seconds': record_object.retry,
+                                u'refresh_seconds': record_object.refresh,
+                                u'expiry_seconds': record_object.expire,
+                                u'minimum_seconds': record_object.minimum}
+
+          else:
+            raise errors.UnexpectedDataError(
+                'Unkown record type: %s.\n %s' % (
+                    dns.rdatatype.to_text(record_object.rdtype), record_object))
+
+          view_dependency = u'any'
+          if( record_object.rdtype == dns.rdatatype.SOA ):
+            if( zone_view ):
+              view_dependency = zone_view
+          else:
+            if( view ):
+              view_dependency = view
+
+          make_record_args_list.append(
+              {u'record_type': record_type,
+               u'record_target': record_target,
+               u'record_zone_name': zone_name,
+               u'record_arguments': record_args_dict,
+               u'record_view_dependency': view_dependency,
+               u'ttl': ttl})
+
+    self.ProcessRecordsBatch(add_records=make_record_args_list)
+
+    return len(make_record_args_list)
+
+  def GetCIDRBlocksByView(self, view, username):
+    """Lists CIDR blocks available to a username in a given view
 
     Inputs:
         view: string of view name
-        user: string of user name
+        username: string of user name
 
     Outputs:
         list: list of cidr block strings
     """
+    self.user_instance.Authorize('GetCIDRBlocksByView')
     cidrs = set([])
     users_dict = self.db_instance.GetEmptyRowDict('users')
-    users_dict['user_name'] = user
+    users_dict['user_name'] = username
     views_dict = self.db_instance.GetEmptyRowDict('views')
     views_dict['view_name'] = view
 
@@ -151,6 +286,7 @@ class CoreHelpers(object):
       self.db_instance.EndTransaction()
     for cidr_dict in joined_list:
       cidrs.add(cidr_dict['reverse_range_zone_assignments_cidr_block'])
+
     return cidrs
 
   def GetAssociatedCNAMEs(self, hostname, view_name, zone_name,
@@ -165,6 +301,7 @@ class CoreHelpers(object):
     Outputs:
       list: list of found cname dictionaries
     """
+    self.user_instance.Authorize('GetAssociatedCNAMEs')
     record_arguments_record_assignments_dict = (
         self.db_instance.GetEmptyRowDict(
             'record_arguments_records_assignments'))
@@ -218,6 +355,7 @@ class CoreHelpers(object):
     cnames.extend(new_cnames)
     del new_cnames
     del found_records
+
     return cnames
 
   def ListLatestNamedConfig(self, dns_server_set):
@@ -251,17 +389,22 @@ class CoreHelpers(object):
     Inputs:
       option_id: the id of config to replicate
       dns_server_set: string of dns server set name
+    
+    Raises:
+      InvalidInputError: DNS server set does not contain id.
+      UnexpectedDataError: Multiple configurations found.
     """
     named_config = self.core_instance.ListNamedConfGlobalOptions(
         dns_server_set=dns_server_set, option_id=option_id)
     if( len(named_config) == 0 ):
-      raise errors.CoreError('DNS server set "%s" does not contain id "%s"' % (
-          dns_server_set, option_id))
+      raise errors.InvalidInputError(
+        'DNS server set "%s" does not contain id "%s"' % (
+            dns_server_set, option_id))
     elif( len(named_config) == 1 ):
       self.core_instance.MakeNamedConfGlobalOption(
           dns_server_set, named_config[0]['options'])
     else:
-      raise errors.CoreError('Multiple configurations found.')
+      raise errors.UnexpectedDataError('Multiple configurations found.')
 
   def MakeAAAARecord(self, target, zone_name, record_args_dict,
                      view_name=None, ttl=None):
@@ -279,13 +422,30 @@ class CoreHelpers(object):
     self.core_instance.MakeRecord(u'aaaa', target, zone_name, record_args_dict,
                                   view_name, ttl)
 
+  def MakeSubdomainDelegation(self, zone_name, subdomain_name, nameserver, 
+                              view_name=u'any'):
+    """"Makes a Delegated Subdomain
+    Assumes delegation zone is created
+
+    Inputs:
+      view_name: string of view name
+      zone_name: string of zone name
+      subdomain_name: string of subdomain name 
+      nameserver: string of fully qualified nameserver
+    Raises:
+      InvalidInputError: Zone does not exist.
+    """
+    self.core_instance.MakeRecord(u'ns', subdomain_name, zone_name,
+                                  {u'name_server':nameserver}, view_name)
+
   def GetPTRTarget(self, long_target, view_name=u'any'):
     """Gets the short PTR target given the long PTR target
-
     Inputs:
       long_target: String of long PTR target
       view_name: String of view name
-
+    
+    Raises:
+      InvalidInputError: No suitable reverse range zone assignments found.
     Ouptuts:
       string: String of short PTR target
     """
@@ -302,7 +462,7 @@ class CoreHelpers(object):
             reverse_range_zone_assignments[zone_assignment]) ):
           break
     else:
-      raise errors.CoreError(
+      raise errors.InvalidInputError(
           'No suitable reverse range zone assignments found.')
     zone_detail = self.core_instance.ListZones(view_name=view_name)
     zone_origin = zone_detail[zone_assignment][view_name]['zone_origin']
@@ -343,7 +503,7 @@ class CoreHelpers(object):
     if( record_args_dict['assignment_host'].startswith('@.') ):
       record_args_dict['assignment_host'] = record_args_dict[
           'assignment_host'].lstrip('@.')
-    self.core_instance.RemoveRecord(u'ptr', target, zone_name,
+    self.core_instance.RemoveRecord(record_type, target, zone_name,
                                     record_args_dict, view_name, ttl)
 
   def ListAvailableIpsInCIDR(self, cidr_block, num_ips=1, view_name=None,
@@ -354,14 +514,18 @@ class CoreHelpers(object):
 
     Inputs:
       cidr_block: string of ipv4 or ipv6 cidr block
-
+    
+    Raises:
+      InvalidInputError: IP is in a reserved IP space.
+      InvalidInputError: Not a valid cidr block
     Outputs:
       list: list of strings of ip addresses
     """
     try:
       cidr_block_ipy = IPy.IP(cidr_block)
     except ValueError:
-      raise errors.CoreError('%s is not a valid cidr block' % cidr_block)
+      raise errors.InvalidInputError(
+          '%s is not a valid cidr block' % cidr_block)
     reserved_ips = []
     if( cidr_block_ipy.version() == 6 ):
       reserved = constants.RESERVED_IPV6
@@ -371,7 +535,8 @@ class CoreHelpers(object):
       reserved_ips.append(IPy.IP(cidr))
     for reserved_ip in reserved_ips:
       if( IPy.IP(cidr_block) in reserved_ip ):
-        raise errors.CoreError('%s is in a reserved IP space' % cidr_block)
+        raise errors.InvalidInputError(
+            '%s is in a reserved IP space' % cidr_block)
     records = self.ListRecordsByCIDRBlock(cidr_block, view_name=view_name,
                                           zone_name=zone_name)
     taken_ips = []
@@ -379,13 +544,14 @@ class CoreHelpers(object):
     for view in records:
       for ip in records[view]:
         taken_ips.append(ip)
-    count = 0L;
+    count = 0L
     while( count < (cidr_block_ipy.len() - 1) ):
       count += 1L
       if( len(avail_ips) >= num_ips ):
         break
       if( cidr_block_ipy[count].strFullsize() not in taken_ips ):
         avail_ips.append(cidr_block_ipy[count].strFullsize())
+
     return avail_ips
 
   def ListRecordsByCIDRBlock(self, cidr_block, view_name=None, zone_name=None):
@@ -395,6 +561,11 @@ class CoreHelpers(object):
       cidr_block: string of ipv4 or ipv6 cidr block
       view_name: string of the view
       zone_name: string of the zone
+    
+    Raise:
+      InvalidInputError: The CIDR block specified does not contain a valid IP
+      IPIndexError: Record type not indexable by IP
+      IPIndexError: Record type unknown. Missing ipv4 or ipv6 dec index
 
     Outputs:
       dict: A dictionary Keyed by view, keyed by IP, listed by record.
@@ -410,6 +581,7 @@ class CoreHelpers(object):
                           u'zone': u'reverse_zone',
                           u'zone_origin': u'1.168.192.in-addr.arpa.'}]}}
     """
+    self.user_instance.Authorize('ListRecordsByCIDRBlock')
     record_list = {}
     try:
       IPy.IP(cidr_block)
@@ -439,14 +611,14 @@ class CoreHelpers(object):
         self.db_instance.GetEmptyRowDict(
             'record_arguments_records_assignments'))
 
-    if( view_name is not None and 
+    if( view_name is not None and
         view_name.endswith('_dep') or view_name == u'any' ):
       records_dict['record_view_dependency'] = view_name
     elif( view_name is not None ):
       records_dict['record_view_dependency'] = '%s_dep' % view_name
-      
+
     zone_dict['zone_name'] = zone_name
-    
+
     if( cidr_ip.version() == 4 ):
       decimal_ip = int( cidr_ip.strDec() )
       decimal_ip_lower = (
@@ -490,7 +662,7 @@ class CoreHelpers(object):
             (decimal_ip_upper >> (64 - cidr_size)) << (64 - cidr_size))
         decimal_ip_upper_upper = (
             (pow(2,64 - cidr_size) - 1 ) | decimal_ip_upper)
-        column='ipv6_dec_upper'
+        column = 'ipv6_dec_upper'
         range_values = (decimal_ip_upper_lower, decimal_ip_upper_upper)
       self.db_instance.StartTransaction()
       try:
@@ -499,16 +671,16 @@ class CoreHelpers(object):
             'records', records_dict,
             'zones', zone_dict,
             'zone_view_assignments', zone_view_assignments_dict,
-            'record_arguments_records_assignments', 
+            'record_arguments_records_assignments',
             record_arguments_records_assignments_dict,
             column=column,
             range_values=range_values)
       finally:
         self.db_instance.EndTransaction()
-     
+
     ## Parse returned list
     parsed_record_dict = {}
-    for index, record_entry in enumerate(record_list):
+    for _, record_entry in enumerate(record_list):
       if( record_entry[u'record_type'] not in
           constants.RECORD_TYPES_INDEXED_BY_IP ):
         raise errors.IPIndexError('Record type not indexable by '
@@ -542,14 +714,15 @@ class CoreHelpers(object):
       record_item['record_ttl'] = record_entry['record_ttl']
       record_item['record_zone_name'] = record_entry['record_zone_name']
       record_item[u'zone_origin'] = record_entry[u'zone_origin']
-      record_item['record_view_dependency'] = record_entry['record_view_dependency']
+      record_item['record_view_dependency'] = record_entry[
+          'record_view_dependency']
       #record_item['record_last_updated'] = record_entry['record_last_updated']
       record_item['record_last_user'] = record_entry['record_last_user']
       if record_entry[u'record_view_dependency'].endswith('_dep'):
         record_item[u'view_name'] = record_entry[u'record_view_dependency'][:-4]
       else:
         record_item[u'view_name'] = record_entry[u'record_view_dependency']
-      if( record_entry[u'record_type'] == u'a' or 
+      if( record_entry[u'record_type'] == u'a' or
           record_entry[u'record_type'] == u'aaaa' ):
         record_item[u'forward'] = True
         record_item[u'host'] = '%s.%s' % (
@@ -566,9 +739,11 @@ class CoreHelpers(object):
         record_item[u'forward'] = False
         record_item[u'host'] = record_entry[u'argument_value'][:-1]
         assignment_ip = helpers_lib.UnReverseIP(
-            '%s.%s' % (record_entry['record_target'],record_entry['zone_origin']))
+            '%s.%s' % (
+                record_entry['record_target'],record_entry['zone_origin']))
         record_item[u'record_args_dict'] = {'assignment_ip': assignment_ip}
         parsed_record_dict[record_view][record_ip].insert(0, record_item )
+
     return parsed_record_dict
 
   def ListNamedConfGlobalOptionsClient(self, option_id=None,
@@ -584,8 +759,16 @@ class CoreHelpers(object):
     Outputs:
       list: list of dictionarires from ListNamedConfGlobalOptions
     """
-    return self.core_instance.ListNamedConfGlobalOptions(
+    function_name, current_args = helpers_lib.GetFunctionNameAndArgs()
+    self.user_instance.Authorize(function_name)
+    success = False
+    named_conf_global_options = None
+
+    named_conf_global_options = self.core_instance.ListNamedConfGlobalOptions(
         option_id, dns_server_set, timestamp)
+    success = True
+
+    return named_conf_global_options
 
   def ListZoneByIPAddress(self, ip_address):
     """Lists zone name given ip_address
@@ -614,12 +797,15 @@ class CoreHelpers(object):
       hostname: string of hostname
       view_name: string of view name
       zone_name: string of zone name
+    
+    Raises:
+      UnexpectedDataError: Incorrect number of records found
 
     Outputs:
       int: number of rows modified
     """
     function_name, current_args = helpers_lib.GetFunctionNameAndArgs()
-
+    row_count = 0
     record_arguments_record_assignments_dict = (
         self.db_instance.GetEmptyRowDict(
             'record_arguments_records_assignments'))
@@ -644,22 +830,24 @@ class CoreHelpers(object):
           remove_record_dict[record_argument[
               'record_arguments_records_assignments_record_id']] = {
                   'assignment_host': record_argument['argument_value']}
-        row_count = 0
         for record_id in remove_record_dict:
           records_dict['records_id'] = record_id
           found_records_dict = self.db_instance.ListRow(
               'records', records_dict)
           if( len(found_records_dict) != 1 ):
-            raise errors.CoreError('Incorrect number of records found!')
+            raise errors.UnexpectedDataError(
+                'Incorrect number of records found!')
           try:
             self.core_instance.user_instance.Authorize(
-                'RemoveRecord',
+                function_name,
                  record_data=
-                     {'target': records_dict['record_target'],
+                     {'target': found_records_dict[0]['record_target'],
                       'zone_name': records_dict['record_zone_name'],
-                      'view_name': records_dict['record_view_dependency']},
+                      'view_name': records_dict['record_view_dependency'],
+                      'record_type': records_dict['record_type']},
+
                 current_transaction=True)
-          except user.AuthError:
+          except errors.AuthorizationError:
             continue
           row_count += self.db_instance.RemoveRow(
               'records', found_records_dict[0])
@@ -670,7 +858,6 @@ class CoreHelpers(object):
         raise
       self.db_instance.EndTransaction()
       success = True
-      remove_record_string = ''
       log_list = []
       for record_id in remove_record_dict:
         log_list.append('record_id:')
@@ -678,14 +865,13 @@ class CoreHelpers(object):
         for record in remove_record_dict[record_id]:
           log_list.append('%s:' % record)
           log_list.append(remove_record_dict[record_id][record])
-      if( log_list ):
-        remove_record_string = ' '.join(log_list)
+      success = True
     finally:
       self.log_instance.LogAction(self.user_instance.user_name, function_name,
                                   current_args, success)
     return row_count
 
-  def ProcessRecordsBatch(self, delete_records=[], add_records=[]):
+  def ProcessRecordsBatch(self, delete_records = None, add_records = None):
     """Proccess batches of records
 
     Inputs:
@@ -698,12 +884,22 @@ class CoreHelpers(object):
                           {'record_type': 'ptr', 'record_target': 'target',
                           'view_name': 'view', 'zone_name': 'zone'}
       add_records: list of dictionaries of records
+    
+    Raises: 
+      RecordsBatchError: Record specification too broad
+      RecordsBatchError: No record found
+      RecordsBatchError: Record already exists
+      RecordsBatchError: CNAME already exists
+      RecordsBatchError: Duplicate record found
 
     Outputs:
       int: row count
     """
+    if delete_records is None:
+      delete_records = []
+    if add_records is None:
+      add_records = []
     function_name, current_args = helpers_lib.GetFunctionNameAndArgs()
-
     log_dict = {'delete': [], 'add': []}
     row_count = 0
     changed_view_dep = []
@@ -714,19 +910,21 @@ class CoreHelpers(object):
         # REMOVE RECORDS
         for record in delete_records:
           record_dict = self.db_instance.GetEmptyRowDict('records')
-          self.user_instance.Authorize('RemoveRecord', 
+          self.user_instance.Authorize('ProcessRecordsBatch',
               record_data = {
                   'target': record['record_target'],
                   'zone_name': record['record_zone_name'],
-                  'view_name': record_dict['record_view_dependency']},
+                  'view_name': record_dict['record_view_dependency'],
+                  'record_type': record['record_type']},
               current_transaction=True)
           record_dict['records_id'] = record['records_id']
           record_dict['record_type'] = record['record_type']
           record_dict['record_target'] = record['record_target']
           record_dict['record_ttl'] = record['record_ttl']
-          if( record['record_view_dependency'].endswith('_dep') or 
+          if( record['record_view_dependency'].endswith('_dep') or
               record['record_view_dependency'] == u'any' ):
-            record_dict['record_view_dependency'] = record['record_view_dependency']
+            record_dict['record_view_dependency'] = record[
+                'record_view_dependency']
           else:
             record_dict['record_view_dependency'] = (
                 '%s_dep' % record['record_view_dependency'])
@@ -750,11 +948,12 @@ class CoreHelpers(object):
           if( not record['record_view_dependency'].endswith('_dep') and record[
                 'record_view_dependency'] != u'any'):
             view_name = '%s_dep' % record['record_view_dependency']
-          self.user_instance.Authorize('MakeRecord', 
+          self.user_instance.Authorize('ProcessRecordsBatch',
               record_data = {
                   'target': record['record_target'],
                   'zone_name': record['record_zone_name'],
-                  'view_name': view_name},
+                  'view_name': view_name,
+                  'record_type': record['record_type']},
               current_transaction=True)
           if( record['record_type'] == u'ptr' ):
             if( record['record_arguments'][
