@@ -66,10 +66,9 @@ Note: MySQLdb.Error can be raised in almost any function in this module. Please
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.15'
+__version__ = '0.16'
 
 
-import datetime
 import Queue
 import threading
 import time
@@ -77,7 +76,6 @@ import uuid
 import warnings
 
 import MySQLdb
-import MySQLdb.cursors
 
 import constants
 import data_validation
@@ -117,11 +115,13 @@ class dbAccess(object):
     self.big_lock_timeout = big_lock_timeout
     self.big_lock_wait = big_lock_wait
     self.ssl = ssl
-    self.ssl_settings = {'ca': ssl_ca, 'cert': ssl_cert, 'key': ssl_key}
-    if( ssl_capath ):
-      self.ssl_settings['capath'] = ssl_capath
-    if( ssl_cipher ):
-      self.ssl_settings['cipher'] = ssl_cipher
+    self.ssl_ca = ssl_ca
+    self.ssl_settings = {}
+    if( self.ssl ):
+      if( self.ssl_ca ):
+        self.ssl_settings['ca'] = ssl_ca
+      else:
+        raise errors.ConfigError('ssl_ca not specified in config file.')
     self.transaction_init = False
     self.connection = None
     self.cursor = None
@@ -155,7 +155,15 @@ class dbAccess(object):
     """
     if( DEBUG == True ):
       print execution_string % values
-    self.cursor.execute(execution_string, values)
+    try:
+      self.cursor.execute(execution_string, values)
+    except MySQLdb.ProgrammingError:
+      raise
+    except MySQLdb.Error, e:
+      if( e[0] in errors.PARSABLE_MYSQL_ERRORS ):
+        raise errors.DatabaseError(e)
+      else:
+        raise
     
     
   def StartTransaction(self):
@@ -171,7 +179,7 @@ class dbAccess(object):
       TransactionError: Cannot start new transaction last transaction not
                         committed or rolled-back.
     """
-    if( self. thread_safe ):
+    if( self.thread_safe ):
       unique_id = uuid.uuid4()
       self.queue.put(unique_id)
 
@@ -199,14 +207,14 @@ class dbAccess(object):
 
     if( self.connection is None ):
       if( self.ssl ):
-          self.connection = MySQLdb.connect(
-              host=self.db_host, user=self.db_user, passwd=self.db_passwd,
-              db=self.db_name, use_unicode=True, charset='utf8',
-              ssl=self.ssl_settings)
+        self.connection = MySQLdb.connect(
+            host=self.db_host, user=self.db_user, passwd=self.db_passwd,
+            db=self.db_name, use_unicode=True, charset='utf8',
+            ssl=self.ssl_settings)
       else:
-          self.connection = MySQLdb.connect(
-              host=self.db_host, user=self.db_user, passwd=self.db_passwd,
-              db=self.db_name, use_unicode=True, charset='utf8')
+        self.connection = MySQLdb.connect(
+            host=self.db_host, user=self.db_user, passwd=self.db_passwd,
+            db=self.db_name, use_unicode=True, charset='utf8')
       self.cursor = self.connection.cursor(MySQLdb.cursors.DictCursor)
 
     while_sleep = 0
@@ -278,6 +286,9 @@ class dbAccess(object):
 
     This function expects for self.db_instance.cursor to be instantiated and
     valid.
+
+    Raises: 
+      TransactionError: Must unlock tables before re-locking them.
     """
     if( self.locked_db is True ):
       raise errors.TransactionError('Must unlock tables before re-locking them')
@@ -293,6 +304,9 @@ class dbAccess(object):
 
     This function expects for self.db_instance.cursor to be instantiated and
     valid. It also expects all tables to be locked.
+
+    Raises:
+      TransactionError: Must lock tables before unlocking them.
     """
     if( self.locked_db is False ):
       raise errors.TransactionError('Must lock tables before unlocking them')
@@ -474,8 +488,19 @@ class dbAccess(object):
                              lock_rows=True)
 
     Raises:
-      InvalidInputError: Table name not valid
       TransactionError: Must run StartTansaction before inserting
+      UnexpectedDataError: If is_date is specified you must specify column and range
+      UnexpectedDataError: If column or range is specified both are needed
+      InvalidInputError: Found unknown option(s)
+      UnexpectedDataError: No args given, must at least have a pair of table name and row dict
+      UnexpectedDataError: Number of unnamed args is not even.
+          Args should be entered in pairs of table name and row dict.
+      InvalidInputError: Table name not valid
+      InvalidInputError: Column not found in row
+      UnexpectedDataError: Column in table is not a DateTime type
+      UnexpectedDataError: Date from range is not a valid datetime object
+      InvalidInputError: Range must be int if is_date is not set
+      InvalidInputError: Multiple tables were passed in but no joins were found
 
     Outputs:
       tuple of row dicts consisting of all the tables that were in the input.
@@ -514,22 +539,23 @@ class dbAccess(object):
         is_date = kwargs['is_date']
         del kwargs['is_date']
       if( column is None and is_date is not None ):
-        raise errors.InvalidInputError('If is_date is specified you must '
+        raise errors.UnexpectedDataError('If is_date is specified you must '
                                        'specify column and range')
       if( bool(column) ^ bool(range_values) ):
-        raise errors.InvalidInputError('If column or range is specified '
+        raise errors.UnexpectedDataError('If column or range is specified '
                                        'both are needed')
       if( kwargs ):
         raise errors.InvalidInputError('Found unknown option(s): '
                                        '%s' % kwargs.keys())
 
     if( not args ):
-      raise errors.InvalidInputError('No args given, must at least have on '
+      raise errors.UnexpectedDataError('No args given, must at least have a '
                                      'pair of table name and row dict')
     if( len(args) % 2 ):
-      raise errors.InvalidInputError('Number of unnamed args is not even. Args '
-                                     'should be entered in pairs of table name '
-                                     'and row dict.')
+      raise errors.UnexpectedDataError(
+          'Number of unnamed args is not even. Args '
+          'should be entered in pairs of table name '
+          'and row dict.')
     count = 0
     for arg in args:
       count += 1
@@ -552,12 +578,13 @@ class dbAccess(object):
 
       if( is_date ):
         if( constants.TABLES[args[0]][column] != 'DateTime' ):
-          raise errors.InvalidInputError('column: %s in table %s is not a'
+          raise errors.UnexpectedDataError('column: %s in table %s is not a'
                                          'DateTime type' % (column, args[0]))
         for date in range_values:
           if( not self.data_validation_instance.isDateTime(date) ):
-            raise errors.InvalidInputError('Date: %s from range is not a valid '
-                                           'datetime object' % date)
+            raise errors.UnexpectedDataError(
+                'Date: %s from range is not a valid '
+                'datetime object' % date)
       else:
         for value in range_values:
           if( not self.data_validation_instance.isUnsignedInt(value) ):
@@ -586,11 +613,11 @@ class dbAccess(object):
     column_names = []
     search_dict = {}
     for table_name, row_dict in tables.iteritems():
-      for k, v in row_dict.iteritems():
-        column_names.append('%s.%s' % (table_name, k))
-        if( v is not None ):
-          search_dict[k] = v
-          query_where.append('%s%s%s%s' % (k, '=%(', k, ')s'))
+      for key, value in row_dict.iteritems():
+        column_names.append('%s.%s' % (table_name, key))
+        if( value is not None ):
+          search_dict[key] = value
+          query_where.append('%s%s%s%s' % (key, '=%(', key, ')s'))
 
     if( range_values ):
       search_dict['start'] = range_values[0]
@@ -631,8 +658,8 @@ class dbAccess(object):
     row_dict = helpers_lib.GetRowDict(table_name)
     if( not row_dict ):
       raise errors.InvalidInputError('Table name not valid: %s' % table_name)
-    for k in row_dict.iterkeys():
-      row_dict[k] = None
+    for key in row_dict.iterkeys():
+      row_dict[key] = None
     return row_dict
 
   # Not sure this is needed, buuuuut.
@@ -651,6 +678,9 @@ class dbAccess(object):
 
     Inputs:
       record_type: string of record type
+    
+    Raises:
+      InvalidInputError: Unknown record type
 
     Outputs:
       dictionary: keyed by argument name with values of data type of that arg
@@ -706,6 +736,7 @@ class dbAccess(object):
 
     Raises:
       InvalidInputError: dict for record type should have these keys
+      FucntionError: No function to check data type
       UnexpectedDataError: Invalid data type
     """
     record_type_dict = self.GetRecordArgsDict(record_type)
@@ -720,7 +751,7 @@ class dbAccess(object):
     for record_arg_name in record_args_dict.keys():
       if( not 'is%s' % record_type_dict[record_arg_name] in
           data_validation_methods ):
-        raise errors.MissingDataTypeError('No function to check data type %s' %
+        raise errors.FucntionError('No function to check data type %s' %
                                           record_type_dict[record_arg_name])
       if( none_ok and record_args_dict[record_arg_name] is None ):
         continue
@@ -823,10 +854,10 @@ class dbAccess(object):
       table_data[table_name]['rows'] = []
       for row in table_rows:
         row_dict = {}
-        for k,v in row.iteritems():
-          row_dict[k] = self.connection.literal(v)
-          if( isinstance(row_dict[k], str) ):
-            row_dict[k] = unicode(row_dict[k],'utf-8')
+        for key, value in row.iteritems():
+          row_dict[key] = self.connection.literal(value)
+          if( isinstance(row_dict[key], str) ):
+            row_dict[key] = unicode(row_dict[key], 'utf-8')
             
 
         table_data[table_name]['rows'].append(row_dict)
