@@ -2,21 +2,21 @@
 
 # Copyright (c) 2009, Purdue University
 # All rights reserved.
-# 
+#
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
-# 
+#
 # Redistributions of source code must retain the above copyright notice, this
 # list of conditions and the following disclaimer.
 #
 # Redistributions in binary form must reproduce the above copyright notice, this
 # list of conditions and the following disclaimer in the documentation and/or
 # other materials provided with the distribution.
-# 
+#
 # Neither the name of the Purdue University nor the names of its contributors
 # may be used to endorse or promote products derived from this software without
 # specific prior written permission.
-# 
+#
 # THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 # AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 # IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -32,53 +32,94 @@
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.15'
+__version__ = '0.16'
 
+PARSABLE_MYSQL_ERRORS = [1452]
 
 class CoreError(Exception):
+  """Error class that all Roster errors are 
+  sub-classed from
+  """
   pass
 
-class AuthError(CoreError):
+class InternalError(CoreError):
+  """Error class that all internal/code errors
+  sub-class from.
+  """
   pass
 
-class ConfigError(CoreError):
+class UserError(CoreError):
+  """Error class that all User errors
+  sub-class from.
+  """
   pass
 
-class DbAccessError(CoreError):
+class AuthenticationError(UserError):
   pass
 
-class InvalidInputError(CoreError):
+class AuthorizationError(UserError):
   pass
 
-class InvalidInputError(DbAccessError):
+class ConfigError(UserError):
   pass
 
-class IPIndexError(CoreError):
+class InvalidInputError(UserError):
   pass
 
-class MaintenanceError(CoreError):
+class ReservedWordError(UserError):
   pass
 
-class MissingDataTypeError(DbAccessError):
+class IPIndexError(InternalError):
   pass
 
-class ReservedWordError(DbAccessError):
+class MaintenanceError(InternalError):
   pass
 
-class RecordError(CoreError):
+class UnexpectedDataError(UserError):
   pass
 
-class RecordsBatchError(CoreError):
+class DatabaseError(UserError):
+  """Raised when DatabaseError is raised and changes its value
+     to be more readable
+  """
+  def __init__(self, value):
+    """Sets instance variable for value"""
+    self.value = value
+  def __str__(self):
+    """Python standard string method, makes value string
+       easier to understand.
+
+    Outputs:
+      str: reformatted error string
+    """
+    array = self.value[1].split(' ')
+    if( self.value[0] == 1452 ):
+      foreign_index = array.index('FOREIGN')
+      attribute = array[-2].strip('(`)')
+      if( attribute == 'view_dependency' ):
+        attribute = 'view'
+      elif( attribute == 'zone_view_assignments' ):
+        attribute = 'zone or view'
+      self.value = "Specified %s does not exist." % attribute
+    return self.value
+
+class RecordError(InternalError):
+  pass
+
+class RecordsBatchError(InternalError):
+  pass
+
+class FunctionError(InternalError):
+  pass
+
+class VersionDiscrepancyError(InternalError):
+  pass
+
+class DbAccessError(InternalError):
   pass
 
 class TransactionError(DbAccessError):
   pass
 
-class UnexpectedDataError(DbAccessError):
+class MissingDataTypeError(DbAccessError):
   pass
-
-class UserError(CoreError):
-  pass
-
-
-# vi: set ai aw sw=2
