@@ -32,7 +32,7 @@
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.15'
+__version__ = '0.16'
 
 
 import inspect
@@ -121,7 +121,7 @@ def ReverseIP(ip_address):
     ip_address: either an ipv4 or ipv6 string
 
   Raises:
-    CoreError: not a valid IP address
+    InvalidInputError: Not a valid IP address.
 
   Outputs:
     string: reverse ip address
@@ -129,13 +129,13 @@ def ReverseIP(ip_address):
   try:
     ip_object = IPy.IP(ip_address)
   except ValueError:
-    raise errors.CoreError('%s is not a valid IP address' % ip_address)
+    raise errors.InvalidInputError('%s is not a valid IP address' % ip_address)
   reverse_ip_string = ip_object.reverseName()
   if( ip_object.version() == 4 ):
     ip_parts = reverse_ip_string.split('.')
     if( '-' in ip_parts[0] ):
-      range = ip_parts.pop(0).split('-')
-      num_ips = int(range[1]) - int(range[0]) + 1
+      ip_range = ip_parts.pop(0).split('-')
+      num_ips = int(ip_range[1]) - int(ip_range[0]) + 1
       netmask = int(32 - (math.log(num_ips) / math.log(2)))
       last_octet = ip_parts.pop(0)
       reverse_ip_string = '.'.join(ip_parts)
@@ -186,14 +186,17 @@ def UnReverseIP(ip_address):
   return new_ip
 
 
-def CIDRExpand(cidr_block):
+def CIDRExpand(cidr_block, begin=None, end=None):
   """Expands a cidr block to a list of ip addreses
+     from begin (integer) to end (integer).
 
   Inputs:
     cidr_block: string of cidr_block
+    begin: integer of ip address to start
+    end: integer of ip address to end
 
   Raises:
-    CoreError: not a valid CIDR block
+    InvalidInputError: Not a valid CIDR block.
 
   Outputs:
     list: list of ip addresses in strings
@@ -201,10 +204,21 @@ def CIDRExpand(cidr_block):
   try:
     cidr_block = IPy.IP(cidr_block)
   except ValueError:
-    raise errors.CoreError('%s is not a valid cidr block' % cidr_block)
+    raise errors.InvalidInputError('%s is not a valid cidr block' % cidr_block)
   ip_address_list = []
-  for ip_address in cidr_block:
-    ip_address_list.append(unicode(ip_address.strFullsize()))
+  end_count = -1
+  length = cidr_block.len()
+  count = 0
+  start_ip = cidr_block.int()
+  if( begin ):
+    start_ip = cidr_block.int() + begin
+  if( begin and end ):
+    end_count = end - begin
+  elif( end ):
+    end_count = end
+  while( count != end_count and count != length and IPy.IP(start_ip + count) in cidr_block ):
+    ip_address_list.append(unicode(IPy.IP(start_ip + count).strFullsize()))
+    count += 1
 
   return ip_address_list
 
@@ -216,7 +230,8 @@ def ExpandIPV6(ip_address):
     ip_address: string of ipv6 address
 
   Raises:
-    CoreError: not a valid IP address
+    InvalidInputError: Not a valid IP address.
+    InvalidInputError: Not a valid IPV6 address.
 
   Outputs:
     string: string of long ipv6 address
@@ -224,9 +239,9 @@ def ExpandIPV6(ip_address):
   try:
     ipv6_address = IPy.IP(ip_address)
   except ValueError:
-    raise errors.CoreError('%s is not a valid IP address' % ip_address)
+    raise errors.InvalidInputError('%s is not a valid IP address' % ip_address)
   if( ipv6_address.version() != 6 ):
-    raise errors.CoreError('"%s" is not a valid IPV6 address.' % ipv6_address)
+    raise errors.InvalidInputError('"%s" is not a valid IPV6 address.' % ipv6_address)
 
   return ipv6_address.strFullsize()
 
