@@ -36,7 +36,7 @@ is handled in this module.
 
 __copyright__ = 'Copyright (C) 2009, Purdue University'
 __license__ = 'BSD'
-__version__ = '0.15'
+__version__ = '0.16'
 
 
 import IPy
@@ -59,7 +59,7 @@ class User(object):
       db_instance:	dbAccess instance to use for user verification
 
     Raises:
-      UserError on non-existent user
+      InvalidInputError: No such user.
     """
     self.user_name = user_name
     self.db_instance = db_instance
@@ -70,7 +70,7 @@ class User(object):
     # pull a pile of authentication info from the database here
     self.user_perms = self.db_instance.GetUserAuthorizationInfo(user_name)
     if( not self.user_perms.has_key('user_name') ):
-      raise errors.UserError("No such user: %s" % user_name)
+      raise errors.InvalidInputError("No such user: %s" % user_name)
 
     # More DB crud
     self.groups = self.user_perms['groups']
@@ -89,17 +89,23 @@ class User(object):
 
     Inputs:
       method:	what the user's trying to do
-      record_data: dictionary of target, zone_name and view_name for 
-                   record that is being modified
+      record_data: dictionary of target, zone_name, view_name, and record_type 
+                   for record that is being modified.
+                   {'target': 'test_target',
+                    'zone_name': 'test_zone',
+                    'view_name': 'test_view',
+                    'record_type': 'a'
+                   }
       current_transaction: bool of if this function is run from inside a 
                            transaction in the db_access class
 
     Raises:
-      UserError if no target is provided (and one is required)
-      AuthError on authorization failure
+      MaintenanceError: Roster is currently under maintenance.
+      MissingDataTypeError: No record data provided for access method.
+      MissingDataTypeError: Incomplete record data provided for access method.
+      AuthorizationError: Authorization failure.
     """
     function_name, current_args = helpers_lib.GetFunctionNameAndArgs()
-    access_level = self.user_perms['user_access_level']
 
     if( not current_transaction ):
       self.db_instance.StartTransaction()
@@ -119,8 +125,9 @@ class User(object):
       raise errors.MaintenanceError('Roster is currently under maintenance.')
 
     if( record_data is not None and record_data.has_key('zone_name') ):
-      target_string = ' with %s on %s' % (record_data['target'],
-                                          record_data['zone_name'])
+      target_string = ' with %s on %s of type %s' % (record_data['target'],
+                                          record_data['zone_name'],
+                                          record_data['record_type'])
     else:
       target_string = ''
     auth_fail_string = ('User %s is not allowed to use %s%s' %
@@ -129,20 +136,29 @@ class User(object):
       method_hash = self.abilities[method]
       if( int(self.user_access_level) >= constants.ACCESS_LEVELS['dns_admin'] ):
         return
+      
       if( method_hash['check'] ):
         # Secondary check - ensure the target is in a range delegated to
         # the user
         if( record_data is None ):
-          raise errors.UserError('No record data provided for access method '
-                                 '%s' % method)
-        if( not record_data.has_key('zone_name') or
+          raise errors.MissingDataTypeError(
+              'No record data provided for access method '
+              '%s' % method)
+        elif( not record_data.has_key('zone_name') or
             record_data['zone_name'] is None or 
             not record_data.has_key('view_name') or
             record_data['view_name'] is None or
             not record_data.has_key('target') or 
-            record_data['target'] is None ):
-          raise errors.UserError('Incomplete record data provided for access '
-                                 'method %s' % method)
+            record_data['target'] is None or
+            not record_data.has_key('record_type') or 
+            record_data['record_type'] is None ):
+          raise errors.MissingDataTypeError(
+              'Incomplete record data provided for access '
+              'method %s' % method)
+          
+        elif( record_data['record_type'] not in constants.USER_LEVEL_RECORDS and
+            int(self.user_access_level) <= constants.ACCESS_LEVELS['user'] ):
+          raise errors.AuthorizationError(auth_fail_string)
 
         for zone in self.forward_zones:
           if( record_data['zone_name'] == zone['zone_name'] ):
@@ -150,7 +166,6 @@ class User(object):
         # Can't find it in forward zones, maybe it's a reverse, lets try to
         # construct an ip address
        
-        
         ip_address = helpers_lib.UnReverseIP('%s.%s' % (
           record_data['target'][::-1], self.zone_origin_cache[
                 record_data['zone_name']]))
@@ -165,20 +180,20 @@ class User(object):
           # fail to find a matching IP range with appropriate perms
           self.log_instance.LogAction(self.user_name, function_name,
                                       current_args, False)
-          raise errors.AuthError(auth_fail_string)
+          raise errors.AuthorizationError(auth_fail_string)
 
         except ValueError:
           # fail to find a matching zone with appropriate perms
           self.log_instance.LogAction(self.user_name, function_name,
                                       current_args, False)
-          raise errors.AuthError(auth_fail_string)
+          raise errors.AuthorizationError(auth_fail_string)
       else:
         return
     else:
       # fail to find a matching method
       self.log_instance.LogAction(self.user_name, function_name,
                                   current_args, False)
-      raise errors.AuthError(auth_fail_string)
+      raise errors.AuthorizationError(auth_fail_string)
 
   def GetUserName(self):
     """Return user name for current session.
